<?php

namespace Hrm\Doc\Policies;

use Litepie\User\Contracts\UserPolicy;
use Hrm\Doc\Models\Doc;

class DocPolicy
{

    /**
     * Determine if the given user can view the doc.
     *
     * @param UserPolicy $user
     * @param Doc $doc
     *
     * @return bool
     */
    public function view(UserPolicy $user, Doc $doc)
    {
        if ($user->canDo('doc.doc.view') && $user->isAdmin()) {
            return true;
        }

        return $doc->user_id == user_id() && $doc->user_type == user_type();
    }

    /**
     * Determine if the given user can create a doc.
     *
     * @param UserPolicy $user
     * @param Doc $doc
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('doc.doc.create');
    }

    /**
     * Determine if the given user can update the given doc.
     *
     * @param UserPolicy $user
     * @param Doc $doc
     *
     * @return bool
     */
    public function update(UserPolicy $user, Doc $doc)
    {
        if ($user->canDo('doc.doc.edit') && $user->isAdmin()) {
            return true;
        }

        return $doc->user_id == user_id() && $doc->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given doc.
     *
     * @param UserPolicy $user
     * @param Doc $doc
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Doc $doc)
    {
        return $doc->user_id == user_id() && $doc->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given doc.
     *
     * @param UserPolicy $user
     * @param Doc $doc
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Doc $doc)
    {
        if ($user->canDo('doc.doc.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given doc.
     *
     * @param UserPolicy $user
     * @param Doc $doc
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Doc $doc)
    {
        if ($user->canDo('doc.doc.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
